#!/usr/bin/env bash
#
# Usage: create_resource.sh cluster_name aws_region namespace
#
readonly cluster_name=$1
readonly aws_region=$2
readonly namespace=$3
readonly service_account_prefix=$4

: \
  ${LUNA_MANAGER_REPO:="elotl/luna-trial"} \
  ${LUNA_WEBHOOK_REPO:="elotl/luna-admission-webhook-trial"}

die() {
    if [ -z "$*" ]
    then
        echo FAIL
    else
        echo FAIL "$*"
    fi
    exit 1
}

_aws() {
    env AWS_PAGER= aws --region="$aws_region" "$@"
}

_eksctl() {
  eksctl --region="$aws_region" "$@"
}

get_policy_arn_from_name() {
  _aws iam list-policies --query 'Policies[*].[PolicyName,Arn]' --output text |
    grep "$1" |
    awk '{ print $2 }'
}

function setup_namespace () {
    output="$(kubectl get namespace "$namespace" 2>&1)"
    if [[ $? -ne 0 ]] ; then
        echo "Creating cluster namespace $namespace"
        kubectl create namespace "$namespace"
    fi
}

oidc_provider_url="$(
  _aws eks describe-cluster \
    --name "$cluster_name" \
    --query 'cluster.identity.oidc.issuer' \
    --output text || die
)"
readonly oidc_provider_url
readonly oidc_provider_base=${oidc_provider_url#https://}
oidc_provider_arn=$(
  _aws iam list-open-id-connect-providers --output text |
  grep "$oidc_provider_base" |
  cut -f 2
)

if [ -z "$oidc_provider_arn" ]
then
    # Verify eksctl is installed
    if [ -z "$(command -v eksctl)" ]
    then
        echo 'ERROR: eksctl not found. Install eksctl and re-run this script.'
        exit 1
    fi

    printf "IAM OpenID connect provider not found; creating one for %s...\n" "$cluster_name" > /dev/stderr
    _eksctl utils associate-iam-oidc-provider --cluster "$cluster_name" --approve || die
    oidc_provider_arn=$(
      _aws iam list-open-id-connect-providers --output text |
      grep "$oidc_provider_base" |
      cut -f 2
    )
    if [ -z "$oidc_provider_arn" ]
    then
		die 'Empty OpenID provider ARN'
    fi
fi

_aws ec2 describe-instances \
  --filter 'Name=instance-state-name,Values=running' \
  "Name=tag:kubernetes.io/cluster/${cluster_name},Values=owned" \
  --query 'Reservations[].Instances[].SecurityGroups[*].GroupId' \
  --output text |
sed 's/\t/\n/g'| sort -u | while read -r sg_id
do
    printf 'Tagging %s\n' "$sg_id" > /dev/stderr
    _aws ec2 create-tags \
      --resources "$sg_id" \
      --tags "Key=elotl.co/nodeless-cluster/name,Value=${cluster_name}" || die
done

tmpdir=$(mktemp -d)
readonly tmpdir
readonly oidc_assume_role_policy="$tmpdir/oidc_assume_role_policy.json"
cat <<EOF > "$oidc_assume_role_policy"
{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "",
            "Effect": "Allow",
            "Principal": {
                "Federated": "${oidc_provider_arn}"
            },
            "Action": "sts:AssumeRoleWithWebIdentity",
            "Condition": {
                "StringLike": {
                    "${oidc_provider_base}:sub": "system:serviceaccount:${namespace}:${service_account_prefix}-manager"
                }
            }
        }
    ]
}
EOF

set +e
readonly create_policy_stderr="$tmpdir/create_policy.stderr"
readonly policy_name="$cluster_name-$namespace-LunaAssumeRoleWithWebIdentity"
readonly create_role_stderr="$tmpdir/create_role.stderr"
readonly role_name="$cluster_name-$namespace-node_manager"

policy_arn=$(get_policy_arn_from_name "$policy_name")

if [ -n "$policy_arn" ]
then
  _aws iam detach-role-policy \
    --role-name "$role_name" \
    --policy-arn "$policy_arn" || true
  _aws iam delete-role --role-name "$role_name" || true
  _aws iam delete-policy --policy-arn "$policy_arn" || true
fi

_aws iam create-policy \
  --policy-name "$policy_name" \
  --policy-document "file://role_policy.json" \
  --output text \
  2>"$create_policy_stderr"

_aws iam create-role \
  --role-name "$role_name" \
  --assume-role-policy-document "file://$oidc_assume_role_policy" \
  --tags 'Key=ServiceAccountName,Value=elotl-luna-manager' \
  --output text \
  2>"$create_role_stderr"

policy_arn=$(get_policy_arn_from_name "$policy_name")

_aws iam attach-role-policy \
  --role-name "$role_name" \
  --policy-arn "$policy_arn"

# Create helm values file
instance_profile_arn=$(
    _aws ec2 describe-instances \
      --filter 'Name=instance-state-name,Values=running' \
      "Name=tag:kubernetes.io/cluster/${cluster_name},Values=owned" \
      --query 'Reservations[0].Instances[0].IamInstanceProfile.Arn' \
      --output text |
      sed 's#arn:aws:iam::[0-9]*:instance-profile\/##'
)
readonly instance_profile_arn
role_arn=$(
  _aws iam get-role --role-name "$role_name" --query 'Role.Arn' --output text
)
readonly role_arn

setup_namespace

readonly helm_values="./${cluster_name}_values.yaml"

cat <<EOF > "$helm_values"
cloudProvider: aws
clusterID: ${cluster_name}
aws:
  lunaManagerRoleArn: ${role_arn}
  nodeInstanceProfile: ${instance_profile_arn}
labels: "elotl-luna=true"
imageRepo: ${LUNA_MANAGER_REPO}
webhookRepo: ${LUNA_WEBHOOK_REPO}
serviceAccount:
  prefix: ${service_account_prefix}
EOF

echo "all resources created"
