#!/usr/bin/env bash

set -e

CLUSTER_PROJECT_ID=""
CLUSTER_NAME=""
CLUSTER_LOCATION=""
LUNA_NODEPOOL=""
HELM_VALUES=""

RELEASE_NAME="elotl-luna"          # Default, can be overridden by --helm-release-name
CLUSTER_NAMESPACE="elotl"          # Default, can be overridden by --namespace
LUNA_SERVICE_ACCOUNT_PREFIX="luna" # Default, can be overridden by --service-account-prefix

CHART_PATH=${CHART_PATH:-"./elotl-luna-v*.tgz"}

cat <<EOF
Thank you for deploying and testing Luna!
Your participation is invaluable to us. If you have any feedback, questions,
or simply want to engage further, please don’t hesitate to join our
Slack channel:
https://join.slack.com/t/elotl-free-trial/shared_invite/zt-1tciz8cck-H9Swzl2grCqPaLJeHYtbBQ

Or email us at info@elotl.co.

Your input helps us improve and refine our product.
EOF

# Check if 'gcloud' is available in the PATH
if ! command -v gcloud &> /dev/null; then
    echo "Error: 'gcloud' command not found."
    echo "Please install the Google Cloud SDK and ensure it is in your PATH."
    exit 1
fi

# Function to display an error message and exit
display_error() {
    echo "Error: $1"
    exit 1
}

# Function to display the script usage
display_usage() {
  echo "Usage: $0 --name <cluster-name> --project <project-id> (--zone <compute-zone> or --region <compute-region>) --node-pool <node-pool> [--helm-release-name <release-name>] [--namespace <namespace>] [--service-account-prefix <prefix>] [--enable-workload-identity] [--additional-helm-values \"<additional-helm-values>\"]"
    exit 1
}

# Function to parse command line arguments
parse_arguments() {
    while [[ $# -gt 0 ]]; do
        key="$1"
        case $key in
            --name)
                shift
                if [[ -z $1 ]]; then
                    display_error "Cluster name not provided for --name option."
                fi
                CLUSTER_NAME="$1"
                ;;
            --project)
                shift
                if [[ -z $1 ]]; then
                    display_error "Cluster project id name not provided for --project option."
                fi
               CLUSTER_PROJECT_ID="$1"
                ;;
            --zone)
                shift
                if [[ -z $1 ]]; then
                  display_error "Cluster location (zone) not provided for --zone option."
                fi

                # Check if the given location is valid
                available_zone_locations=$(gcloud compute zones list --format=value\(name\))
                if ! grep -q "^$1$" <<< "$available_zone_locations"; then
                  display_error "Cluster location provided ($1) for --zone option is not valid."
                fi

                CLUSTER_LOCATION="$1"
                ;;
              --region)
                shift
                if [[ -z $1 ]]; then
                  display_error "Cluster location (region) not provided for --region option."
                fi

                # Check if the given location is valid
                available_regions_locations=$(gcloud compute regions list --format=value\(name\))
                if ! grep -q "^$1$" <<< "$available_regions_locations"; then
                  display_error "Cluster location provided ($1) for --region option is not valid."
                fi

                CLUSTER_LOCATION="$1"
                ;;
            --node-pool)
                shift
                if [[ -z $1 ]]; then
                    display_error "Node pool not provided for --node-pool option."
                fi
                LUNA_NODEPOOL="$1"
                ;;
            --helm-release-name)
                shift
                if [[ -z $1 ]]; then
                    display_error "Release name not provided for --helm-release-name option."
                fi
                RELEASE_NAME="$1"
                ;;
            --namespace)
                shift
                if [[ -z $1 ]]; then
                    display_error "Namespace not provided for --namespace option."
                fi
                CLUSTER_NAMESPACE="$1"
                ;;
            --service-account-prefix)
                shift
                if [[ -z $1 ]]; then
                    display_error "Service Account Prefix not provided for --service-account-prefix option."
                fi
                LUNA_SERVICE_ACCOUNT_PREFIX="$1"
                ;;
            --enable-workload-identity)
                export ENABLE_WORKLOAD_IDENTITY="true"
                ;;
            --additional-helm-values)
                shift
                if [[ -z $1 ]]; then
                    display_error "Additional Helm values not provided for --additional-helm-values option."
                fi
                HELM_VALUES="$1"
                ;;
            --help)
                display_usage
                ;;
            *)
                display_error "Unrecognized option: $key"
                ;;
        esac
        shift
    done
}

# Function to validate the required options
validate_options() {
  if [[ (-z $CLUSTER_NAME || -z $CLUSTER_PROJECT_ID || -z $CLUSTER_LOCATION || -z $LUNA_NODEPOOL || -z $RELEASE_NAME || -z $CLUSTER_NAMESPACE ) ]]; then
        display_usage
    fi
}

# Function to display the values of the variables
display_values() {
    echo "Cluster Name:             $CLUSTER_NAME"
    echo "Cluster Project:          $CLUSTER_PROJECT_ID"
    echo "Cluster Location:         $CLUSTER_LOCATION"
    echo "Node Pool:                $LUNA_NODEPOOL"
    if [[ $ENABLE_WORKLOAD_IDENTITY != "true" ]]
    then
      echo "Enable Workload Identity: (prompt to confirm)"
    else
      echo "Enable Workload Identity: $ENABLE_WORKLOAD_IDENTITY"
    fi
    echo "Helm Release Name:        $RELEASE_NAME"
    echo "Namespace:                $CLUSTER_NAMESPACE"
    echo "Additional Helm Values:   $HELM_VALUES"
    echo "Service Account Prefix:   $LUNA_SERVICE_ACCOUNT_PREFIX"
}

# Function to prompt the user to verify the values
prompt_verification() {
    if [[ -t 1 ]]; then
        echo "Please verify the values displayed above."
        while true; do
            read -rp "Are the values correct? (Y/N): " choice
            case $choice in
                [Yy])
                    break
                    ;;
                [Nn])
                    echo "Script execution canceled by the user."
                    exit 1
                    ;;
                *)
                    echo "Invalid input. Please enter Y or N."
                    ;;
            esac
        done
    fi
}

extract_helm_value() {
    local pattern="$1"

    # Remove leading and trailing whitespace
    input_string=$(echo "$HELM_VALUES" | xargs)

    # Find the substring that matches the pattern
    substring=$(echo "$input_string" | grep -oE -- "$pattern=[^[:space:]]+")

    if [ -n "$substring" ]; then
        # Extract the value
        value=$(echo "$substring" | cut -d= -f2)
        echo "$value"
    fi
}

no_helm_chart_installed() {
    if helm -n ${CLUSTER_NAMESPACE} status ${RELEASE_NAME} >/dev/null 2>&1
    then
        cat <<EOF
Upgrading Luna from this script isn’t supported. Please run the following w/<retained-values-file> set to <retained-path>/${CLUSTER_NAME}_values_full.yaml, for post Luna 0.5.4 installation, or <retained-path>/${CLUSTER_NAME}_values.yaml otherwise:

$ helm upgrade ${RELEASE_NAME} $CHART_PATH --wait --namespace=${CLUSTER_NAMESPACE} --values=<retained-values-file> $HELM_VALUES
EOF
        exit 1
    fi
}

deploy_cert_manager_if_needed() {
  if kubectl wait pods -A -l app.kubernetes.io/instance=cert-manager --for=condition=ready &>/dev/null ; then
    echo "detected cert-manager running"
  else
    echo "no cert-manager deployment found, installing a new one"
    kubectl apply -f https://github.com/cert-manager/cert-manager/releases/download/v1.11.1/cert-manager.yaml &>/dev/null
    if kubectl wait pods -A -l app.kubernetes.io/instance=cert-manager --for=condition=ready --timeout=5m &>/dev/null ; then
      echo "successfully installed cert-manager"
    else
      echo "unable to install cert-manager, aborting"
      exit 1
    fi
  fi
}

deploy() {
    echo "deploying changes to namespace ${CLUSTER_NAMESPACE} in ${CLUSTER_PROJECT_ID} GCP project"
    echo "Creating resources..."
    export LUNA_SERVICE_ACCOUNT_PREFIX
    ./create_resources.sh "$CLUSTER_PROJECT_ID" "$CLUSTER_NAME" "$CLUSTER_NAMESPACE" "$CLUSTER_LOCATION" "$LUNA_NODEPOOL" "$LUNA_SERVICE_ACCOUNT_PREFIX"
    echo "deploying luna-manager helm chart in namespace $CLUSTER_NAMESPACE"
    echo "Excluding deamonsets fluentbit-gke-256pd, fluentbit-gke-max, and gke-metrics-agent-scaling-500 from the resource allocation calculation."
    echo "You can override this exclusion list by setting daemonSetExclude in [helm_values]"
    helm install "$RELEASE_NAME" $CHART_PATH \
        --wait \
        --create-namespace \
        --namespace="${CLUSTER_NAMESPACE}" \
        --values="${CLUSTER_NAME}"_values.yaml \
        --set daemonSetExclude="fluentbit-gke-256pd\,fluentbit-gke-max\,gke-metrics-agent-scaling-500" \
        --set serviceAccount.prefix="${LUNA_SERVICE_ACCOUNT_PREFIX}" \
        $HELM_VALUES
    HELM_fullnameOverride=$(extract_helm_value "fullnameOverride")
    FULL_RELEASE_NAME=""
    if [ ${RELEASE_NAME} = "elotl-luna" ] || [ "${HELM_fullnameOverride}" != "" ]
    then
      FULL_RELEASE_NAME=${RELEASE_NAME}
    else
      FULL_RELEASE_NAME=${RELEASE_NAME}-elotl-luna
    fi
    echo "Saving helm values to ${CLUSTER_NAME}_${FULL_RELEASE_NAME}_values_full.yaml"
    kubectl get cm ${FULL_RELEASE_NAME} -n ${CLUSTER_NAMESPACE} -o "jsonpath={.data['config\.yml']}" > ${CLUSTER_NAME}_${FULL_RELEASE_NAME}_values_full.yaml
}

parse_arguments "$@"
validate_options
display_values
prompt_verification
no_helm_chart_installed
deploy_cert_manager_if_needed
deploy
